/*
 * Copyright 1993, 1995, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* cutout.c
 * 	This program demonstrates how to use multiple  
 * 	clipping planes to cut a hole out of an object.
 *
 * 	ESCAPE key	- exits program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  animate( GLvoid );
GLvoid  visibility( GLint );
GLvoid  keyboard( GLubyte, GLint, GLint );

GLvoid  drawTorusRing( GLboolean );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLfloat angle = 0.0; 

GLvoid
main( int argc, char *argv[] )
{
	GLsizei  width, height, winSize;

	glutInit( &argc, argv );

	width = glutGet(GLUT_SCREEN_WIDTH); 
	height = glutGet(GLUT_SCREEN_HEIGHT);
	winSize = ( width < height ) ? width : height;
	glutInitWindowPosition( winSize/4, winSize/4 ); 
	glutInitWindowSize( winSize/2, winSize/2 );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutIdleFunc( animate ); 
	glutVisibilityFunc( visibility ); 
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );
	glutMainLoop();
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates how to use multiple "
		"clipping planes\nto create a cutout in a sphere.\n\n"
		"Escape key	- exit the program \n\n",
		progname );
}

void
initgfx( void )
{
	GLfloat front_diffuse[] = { 1.0, 0.1, 0.1, 1.0};
	GLfloat back_diffuse[] = { 0.1, 1.0, 0.1, 1.0};
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 40.0 };

	glMaterialfv(GL_FRONT, GL_DIFFUSE, front_diffuse);
	glMaterialfv(GL_BACK, GL_DIFFUSE, back_diffuse);

	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);

	glEnable( GL_LIGHTING );
	glEnable( GL_LIGHT0 );

	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 ); 
}

GLvoid
animate( GLvoid )
{
	/* update the current angle */
	angle = fmodf( (angle - 1.0), 360.0 );

	/* Tell GLUT to redraw the scene */
	glutPostRedisplay();
}
        
GLvoid
visibility( int state )
{
	if (state == GLUT_VISIBLE) {
		glutIdleFunc( animate );
	} else {
		glutIdleFunc( NULL );
	}
}

GLvoid
drawScene( GLvoid )
{
	/* clips away all but right hand wedge when enabled */
	GLdouble right[4] = { 1.0, 0.0, 0.0, -0.5 };
	/* clips away all but left hand wedge when enabled */
	GLdouble left[4]  = { -1.0, 0.0, 0.0, -0.5 };
	GLdouble bottom[4] = { 0.0, 1.0, 0.0, -0.5 };
	GLdouble top[4]  = { 0.0, -1.0, 0.0, -0.5 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix(); 
		glRotatef( angle, 0.0, 1.0, 0.0 );

		/* define clipping planes fixed in scene */
		glClipPlane (GL_CLIP_PLANE0, left);
		glClipPlane (GL_CLIP_PLANE1, right);
		glClipPlane (GL_CLIP_PLANE2, top);
		glClipPlane (GL_CLIP_PLANE3, bottom);
		
		/* clip parts of the solid sphere */
		glEnable( GL_CLIP_PLANE0 );
			glutSolidSphere(1.5, 15, 15);
		glDisable( GL_CLIP_PLANE0 );

		glEnable( GL_CLIP_PLANE1 );
			glutSolidSphere(1.5, 15, 15);
		glDisable( GL_CLIP_PLANE1 );

		glEnable( GL_CLIP_PLANE2 );
			glutSolidSphere(1.5, 15, 15);
		glDisable( GL_CLIP_PLANE2 );

		glEnable( GL_CLIP_PLANE3 );
			glutSolidSphere(1.5, 15, 15);
		glDisable( GL_CLIP_PLANE3 );

		glutSolidSphere(0.3, 15, 15);

	glPopMatrix();

	glutSwapBuffers();
}
