/*
 * Copyright 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* make_current_read.c
 * This program is a simplistic example of how to use 
 * glXMakeCurrentReadSGI.  The second window uses the first
 * window as its read source.
 * 
 *	<c> key 	- copy from read window to write window
 *	Escape key	- exit program
 */
/* compile: cc -o make_current_read make_current_read.c -lGLU -lGL -lX11 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define WIDTH 200
#define HEIGHT 200

static Window win[2];
static int widths[2], heights[2];

static GLfloat clearColors[2][3] = { { 0, 1, 0 } , { 1, 0, 0 } };

static GLXContext ctxt;

void initgfx(void)
{
	glEnable(GL_SCISSOR_TEST);
	glEnable(GL_DEPTH_TEST);

	glViewport(10, 10, WIDTH-20, HEIGHT-20);
	glScissor(10, 10, WIDTH-20, HEIGHT-20);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -0.5, 1000.0);
	glMatrixMode(GL_MODELVIEW);

	glClearColor(0.0, 0.0, 0.0, 0.0);

	glDisable(GL_POLYGON_SMOOTH);
	glDisable(GL_BLEND);
}

void draw(int windex)
{
	glClearColor(clearColors[windex][0],clearColors[windex][1],
			clearColors[windex][2], 1.0);
	glClear(GL_COLOR_BUFFER_BIT);

	glColor3f(0.0, 1.0, 0.0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glBegin(GL_TRIANGLES);
	glVertex3f( 0.9, -0.9, -30.0);
	glVertex3f( 0.9,  0.9, -30.0);
	glVertex3f(-0.9,  0.0, -30.0);
	glEnd();
	glFlush();

	glColor3f(0.0, 0.0, 1.0);

	glBegin(GL_TRIANGLES);
	glVertex3f(-0.9, -0.9, -40.0);
	glVertex3f(-0.9,  0.9, -40.0);
	glVertex3f( 0.9,  0.0, -25.0);
	glEnd();
	glFlush();
}

void
main(int argc, char **argv)
{
	XSetWindowAttributes swa;
	XWindowAttributes gwa;
        Visual *vis;
        XVisualInfo *visualInfo;
        Colormap cmap;
	Display *dpy;
	int             major, minor;

	int attribList[] = { GLX_RGBA, GLX_DEPTH_SIZE, 1, None, };

	int i, width, height;
	char buf[100], *s = NULL;
	XEvent event;
	KeySym ks;
	GLboolean cont = GL_TRUE;
	GLboolean refreshWrite = GL_TRUE;
	GLboolean refreshRead = GL_TRUE;

	int drawWin = 0, readWin = 1;

	dpy = XOpenDisplay(0);
	if (!dpy) {
		fprintf(stderr, "Can't connect to display\n");
		return;
	}

        visualInfo = glXChooseVisual(dpy, DefaultScreen(dpy), attribList);
        if (!visualInfo) {
                fprintf(stderr, "Can't find suitable visual\n");
                exit(-1);
        }
        vis = visualInfo->visual;

        cmap = XCreateColormap(dpy, RootWindow(dpy, DefaultScreen(dpy)), 
				vis, AllocNone);

        /* Create first window */
        swa.colormap = cmap;
        swa.background_pixel = BlackPixel(dpy, DefaultScreen(dpy));
        swa.event_mask = StructureNotifyMask | ExposureMask | KeyPressMask;
        win[0] = XCreateWindow(dpy, RootWindow(dpy, DefaultScreen(dpy)), 
				0, 0, WIDTH, HEIGHT, 0,
				visualInfo->depth, InputOutput, vis, 
				CWBackPixel | CWBorderPixel | CWEventMask | 
				CWColormap, &swa);
        XMapWindow(dpy, win[0]);
	XStoreName(dpy, win[0], "MakeCurrentRead0 - Read Source");

	/* create a GLX context */
	if ((ctxt = glXCreateContext(dpy, visualInfo, 0, GL_TRUE)) == NULL) {
		fprintf(stderr,"Cannot create a context.\n");
		exit(-1);
	}
	XSync(dpy, False); /* Forces the CreateContext to go to the server */

        /* Create second window */
        XGetWindowAttributes(dpy, win[0], &gwa);
        swa.background_pixel = None;
        swa.border_pixel = None;
        swa.colormap = gwa.colormap;
        swa.event_mask = StructureNotifyMask | KeyPressMask | ExposureMask;
        win[1] = XCreateWindow(dpy, gwa.root,
                                0, 0, WIDTH, HEIGHT, 0,
                                gwa.depth, InputOutput, gwa.visual,
                                CWBackPixel | CWBorderPixel | CWColormap |
                                CWEventMask, &swa);
        XStoreName(dpy, win[1], "MakeCurrentRead1 - Draw Window");
        XMapWindow(dpy, win[1]);

#ifdef GLX_SGI_make_current_read
	if ( glXQueryVersion(dpy, &major, &minor) == GL_FALSE ) {
		fprintf(stderr, "glXQueryVersion() failed.\n");
		exit(-1);
	}
#ifdef GLX_VERSION_1_1
	if ( minor > 0 || major > 1 )
		s = glXQueryExtensionsString( dpy, DefaultScreen(dpy) );
#endif
#endif
	if (!s || !strstr(s, "GLX_SGI_make_current_read")) {
		fprintf(stderr, "GLX_SGI_make_current_read not supported\n");
	}

        for (i = 0; i < 2; i++) {
            glXMakeCurrent(dpy, win[i], ctxt);
            glClearColor(clearColors[i][0],clearColors[i][1],
                         clearColors[i][2], 1.0);
            glClear(GL_COLOR_BUFFER_BIT);
            glXSwapBuffers(dpy, win[i]);
        }

#ifdef GLX_SGI_make_current_read
	if (!glXMakeCurrentReadSGI(dpy, win[drawWin], win[readWin], ctxt)) {
		printf("glXMakeCurrentReadSGI failed\n");
		return;
	}
#endif
	glReadBuffer(GL_FRONT);

	initgfx();

	/*
	** Copy when you hit 'c'.
	*/
	while (cont) {
		XNextEvent(dpy, &event);
		switch (event.type) {
		case Expose:
			if (event.xexpose.window == win[drawWin]) {
				refreshWrite = GL_TRUE;
			} else
				refreshRead = GL_TRUE;
			break;
		case ConfigureNotify:
			width = event.xconfigure.width;
			height = event.xconfigure.height;
			if (event.xconfigure.window == win[drawWin]) {
				widths[drawWin] = width;
				heights[drawWin] = height;
				refreshWrite = GL_TRUE;
			} else {
				widths[readWin] = width;
				heights[readWin] = height;
				refreshRead = GL_TRUE;
			}
			break;

		case KeyPress:
			XLookupString(&event.xkey, buf, sizeof(buf), &ks, 0);
			switch (ks) {
			case XK_c:
			case XK_C:
#ifdef GLX_SGI_make_current_read
				glClearColor(clearColors[drawWin][0],
						clearColors[drawWin][1],
						clearColors[drawWin][2], 1.0);
				glClear(GL_COLOR_BUFFER_BIT);
				glRasterPos2i(-1,-1);
				glCopyPixels(0, 0, widths[readWin],
						heights[readWin], GL_COLOR);
				glXSwapBuffers(dpy, win[drawWin]);
#endif
				break;
			case XK_Escape:
				glXMakeCurrent(dpy, None, NULL);
				glXDestroyContext(dpy, ctxt);
				XCloseDisplay(dpy);
				cont = GL_FALSE;
				break;
			}
		}
		if (refreshRead) {
			glXMakeCurrent(dpy, win[readWin], ctxt);
			draw(readWin);
			refreshRead = GL_FALSE;
#ifdef GLX_SGI_make_current_read
			glXMakeCurrentReadSGI(dpy, win[drawWin],
                                      win[readWin], ctxt);
#endif
		} else if (refreshWrite) {
			draw(drawWin);
			refreshWrite = GL_FALSE;
		}
	}
}
