/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* depthcue.c - demonstrates using fog for depth cueing 
 *
 *	Escape key	- exit the program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

void
main( int argc, char *argv[] )
{
	GLsizei 	width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width/4, height/4 ); 
	glutInitWindowSize( width/2, height/2 );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout,
		"\n%s - demonstrates how to use fog for depth cueing\n\n"\
		"Escape key		- exit the program\n\n",
		progname
	);
}

/* Initialize enable depth buffer, and set initial fog parameters  */

GLvoid
initgfx( GLvoid )
{
	GLfloat 	fogColor[4] = { 0.0, 0.0, 0.0, 1.0 };

	glEnable( GL_DEPTH_TEST );

	glFogi( GL_FOG_MODE, GL_LINEAR );
	glHint( GL_FOG_HINT, GL_NICEST );  /*  per pixel   */
	glFogf( GL_FOG_START, 3.0 );
	glFogf( GL_FOG_END, 5.0 );

	glFogfv( GL_FOG_COLOR, fogColor );

	glEnable( GL_FOG );

	/* clear background to the same color as the fog */
	glClearColor( fogColor[0], fogColor[1], fogColor[2], fogColor[3] );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 5.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -4.0 );  /*  move object into view   */
}

GLvoid
drawScene( GLvoid )
{
	int i,  slices = 8;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	glColor3f( 1.0, 1.0, 1.0 );
	glutWireIcosahedron();

	glutSwapBuffers();
}
