/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* edges.c - open a window, clear the background, and render
 *      a non-convex polygon using triangles.  Set the edge flag
 *	to false for nonboundary edges.
 *
 *	<e> Key		- toggle inner edge on/off
 *	Escape key	- exit the program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLboolean showEdges = GL_FALSE;

void
main( int argc, char *argv[] )
{
	GLsizei 	width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width/4, height/4 ); 
	glutInitWindowSize( width/2, height/2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout,
		"\n%s - demonstrates how to set the edge flag so that\n"
		"non-boundary edges are not rendered\n\n"
		"<e> Key		- toggle edge flags on/off\n"
		"Escape key		- exit the program\n\n",
		progname
	);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case 'e':
		showEdges = !showEdges;
		glutPostRedisplay();
		break;
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
        glOrtho( -2.0, 2.0, -2.0, 2.0, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat v0[2] = { -1.0, 1.0 };
	static GLfloat v1[2] = { -1.0, -1.0 };
	static GLfloat v2[2] = { 1.0, -1.0};
	static GLfloat v3[2] = { 1.0, 1.0 };
	static GLfloat v4[2] = { 0.0, 0.0 };
	
	glClear( GL_COLOR_BUFFER_BIT );

	glPolygonMode( GL_FRONT, GL_LINE ); 
	glLineWidth( 2.5 );	/* make lines really wide */
	glColor3f( 1.0, 1.0, 1.0 );

	/* Draw a nonconvex polygon by breaking it up into three triangles */
	glBegin( GL_TRIANGLES );
		/* glEdgeFlag is initially true */
		glVertex2fv( v0 );
		if (!showEdges)
			glEdgeFlag( GL_FALSE );
		glVertex2fv( v1 );
		glEdgeFlag( GL_TRUE );
		glVertex2fv( v4 );

		glVertex2fv( v1 );
		if (!showEdges)
			glEdgeFlag( GL_FALSE );
		glVertex2fv( v2 );
		glVertex2fv( v4 );

		glVertex2fv( v4 );
		glEdgeFlag( GL_TRUE );
		glVertex2fv( v2 );
		glVertex2fv( v3 );
	glEnd();

	glFlush();
}
