/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* lineStipple.c  - open a window, clear the background, 
 *	and render some lines each with different stippling.
 *
 *	Escape key	- exit the program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

void
main( int argc, char *argv[] )
{
	GLsizei 	width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width/4, height/4 ); 
	glutInitWindowSize( width/2, height/2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates line stippling\n\n"
		"Escape key		- exit the program\n\n",
		progname
	);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glLineWidth( 3.5 );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
        glOrtho( 0.0, 1.0, 0.0, 1.0, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT );

	/* Draw lines each with a different stipple factor */
	glColor3f( 1.0, 1.0, 0.0 );
	glBegin( GL_LINE_STRIP );
		glVertex2f( 0.1, 0.1 );
		glVertex2f( 0.5, 0.1 );
		glVertex2f( 0.9, 0.2 );
	glEnd();

	glEnable( GL_LINE_STIPPLE );

	glLineStipple( 1, 0x8888 );
	glBegin( GL_LINE_STRIP );
		glVertex2f( 0.1, 0.3 );
		glVertex2f( 0.5, 0.3 );
		glVertex2f( 0.9, 0.4 );
	glEnd();

	glLineStipple( 2, 0x8888 );
	glBegin( GL_LINE_LOOP );
		glVertex2f( 0.1, 0.5 );
		glVertex2f( 0.5, 0.5 );
		glVertex2f( 0.9, 0.6 );
	glEnd();

	glLineStipple( 3, 0x8888 );
	glBegin( GL_LINE_LOOP );
		glVertex2f( 0.1, 0.7 );
		glVertex2f( 0.5, 0.7 );
		glVertex2f( 0.9, 0.8 );
	glEnd();

	glDisable( GL_LINE_STIPPLE );
	glFlush();
}
