/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* quadrics.c - demonstrates the use of the quadrics
 *  	Utility Library routines to draw circles and arcs.
 *
 *	Escape key	- exit the program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLUquadricObj * quadObj;

void
main( int argc, char *argv[] )
{
	GLsizei 	width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width/4, height/4 ); 
	glutInitWindowSize( width/2, height/2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );
	
	initgfx();

	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates how to draw circles and arcs "
		"using quadrics\n\n"
		"Escape key		- exit the program\n\n",
		progname
	);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 0.0, 1.0 );

	glShadeModel( GL_FLAT );

	/* Create a quadric object */
	quadObj = gluNewQuadric ();
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit when the Escape key is pressed */
		exit(0);
	}
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	aspect = (GLdouble) width / (GLdouble) height;
	if (aspect < 1.0) 
		glOrtho (0.0, 50.0, 0.0, 50.0*(1.0/aspect), -1.0, 1.0);
	else 
		glOrtho (0.0, 50.0*aspect, 0.0, 50.0, -1.0, 1.0);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT );

	glPushMatrix();
		gluQuadricDrawStyle (quadObj, GLU_FILL);
		glColor3f (1.0, 1.0, 1.0);
		glTranslatef (10.0, 10.0, 0.0);
		gluDisk (quadObj, 0.0, 5.0, 10, 2);
	glPopMatrix();

	glPushMatrix();
		glColor3f (1.0, 1.0, 0.0);
		glTranslatef (20.0, 20.0, 0.0);
		gluPartialDisk (quadObj, 0.0, 5.0, 10, 3, 30.0, 120.0);
	glPopMatrix();

	glPushMatrix();
		gluQuadricDrawStyle (quadObj, GLU_SILHOUETTE);
		glColor3f (0.0, 1.0, 1.0);
		glTranslatef (30.0, 30.0, 0.0);
		gluPartialDisk (quadObj, 0.0, 5.0, 10, 3, 135.0, 270.0);
	glPopMatrix();

	glPushMatrix();
		gluQuadricDrawStyle (quadObj, GLU_LINE);
		glColor3f (1.0, 0.0, 1.0);
		glTranslatef (40.0, 40.0, 0.0);
		gluDisk (quadObj, 2.0, 5.0, 10, 10);
	glPopMatrix();


	glFlush();
}
